@@ -4,9 +4,10 @@
 #' @description This function wraps the \code{\link[psych]{describe}}-function
 #'              and prints a basic descriptive statistic, including variable labels.
 #'
-#' @param data A vector or a data frame. May also be a grouped data frame
+#' @param x A vector or a data frame. May also be a grouped data frame
 #'          (see 'Note' and 'Examples').
-#' @inheritParams flat_table
+#'
+#' @inheritParams to_factor
 #'
 #' @return A data frame with basic descriptive statistics, derived from the
 #'         \code{\link[psych]{describe}}-function.
@@ -37,10 +38,10 @@
 #' @importFrom dplyr select mutate
 #' @importFrom psych describe
 #' @export
-descr <- function(data, ...) {
+descr <- function(x, ...) {
 
   # get dot data
-  dd <- get_dot_data(data, match.call(expand.dots = FALSE)$`...`)
+  dd <- get_dot_data(x, match.call(expand.dots = FALSE)$`...`)
 
   # do we have a grouped data frame?
   if (inherits(dd, "grouped_df")) {
