@@ -14,10 +14,12 @@
 #'          groups does not define proper ("equal sized") group sizes.
 #'          See 'Note' and 'Examples'.
 #'
+#' @inheritParams to_factor
 #' @inheritParams group_var
 #' @inheritParams rec
 #'
-#' @return A grouped variable with equal sized groups.
+#' @return A grouped variable with equal sized groups. If \code{x} is a data
+#'         frame, only the grouped variables will be returned.
 #'
 #' @details \code{split_var} splits a variable into equal sized groups, where the
 #'            amount of groups depends on the \code{groupcount}-argument. Thus,
@@ -39,42 +41,63 @@
 #' table(efc$neg_c_7)
 #'
 #' # split into 3 groups
-#' table(split_var(efc$neg_c_7, 3))
+#' table(split_var(efc$neg_c_7, groupcount = 3))
 #'
+#' # split multiple variables into 3 groups
+#' split_var(efc, neg_c_7, pos_v_4, e17age, groupcount = 3)
+#' frq(split_var(efc, neg_c_7, pos_v_4, e17age, groupcount = 3))
 #'
 #' # original
 #' table(efc$e42dep)
 #'
 #' # two groups, non-inclusive cut-point
 #' # vector split leads to unequal group sizes
-#' table(split_var(efc$e42dep, 2))
+#' table(split_var(efc$e42dep, groupcount = 2))
 #'
 #' # two groups, inclusive cut-point
 #' # group sizes are equal
-#' table(split_var(efc$e42dep, 2, inclusive = TRUE))
+#' table(split_var(efc$e42dep, groupcount = 2, inclusive = TRUE))
 #'
 #' @importFrom stats quantile
 #' @export
-split_var <- function(x, groupcount, as.num = FALSE, val.labels = NULL, var.label = NULL, inclusive = FALSE, suffix = "_g") {
-  UseMethod("split_var")
-}
+split_var <- function(x, ..., groupcount, as.num = FALSE, val.labels = NULL, var.label = NULL, inclusive = FALSE, suffix = "_g") {
+  # evaluate arguments, generate data
+  .dots <- match.call(expand.dots = FALSE)$`...`
+  .dat <- get_dot_data(x, .dots)
 
-#' @export
-split_var.data.frame <- function(x, groupcount, as.num = FALSE, val.labels = NULL, var.label = NULL, inclusive = FALSE, suffix = "_g") {
-  tmp <- tibble::as_tibble(lapply(x, FUN = split_var_helper, groupcount, as.num, val.labels, var.label, inclusive))
-  # change variable names, add suffix "_r"
-  if (!is.null(suffix) && !is_empty(suffix)) colnames(tmp) <- sprintf("%s%s", colnames(tmp), suffix)
-  tmp
-}
+  if (is.data.frame(x)) {
 
-#' @export
-split_var.list <- function(x, groupcount, as.num = FALSE, val.labels = NULL, var.label = NULL, inclusive = FALSE, suffix = "_g") {
-  lapply(x, FUN = split_var_helper, groupcount, as.num, val.labels, var.label, inclusive)
-}
+    # iterate variables of data frame
+    for (i in colnames(.dat)) {
+      x[[i]] <- split_var_helper(
+        x = .dat[[i]],
+        groupcount = groupcount,
+        as.num = as.num,
+        var.label = var.label,
+        val.labels = val.labels,
+        inclusive = inclusive
+      )
+    }
 
-#' @export
-split_var.default <- function(x, groupcount, as.num = FALSE, val.labels = NULL, var.label = NULL, inclusive = FALSE, suffix = "_g") {
-  split_var_helper(x, groupcount, as.num, val.labels, var.label, inclusive)
+    # coerce to tibble and select only recoded variables
+    x <- tibble::as_tibble(x[colnames(.dat)])
+
+    # add suffix to recoded variables?
+    if (!is.null(suffix) && !sjmisc::is_empty(suffix)) {
+      colnames(x) <- sprintf("%s%s", colnames(x), suffix)
+    }
+  } else {
+    x <- split_var_helper(
+      x = .dat,
+      groupcount = groupcount,
+      as.num = as.num,
+      var.label = var.label,
+      val.labels = val.labels,
+      inclusive = inclusive
+    )
+  }
+
+  x
 }
 
 split_var_helper <- function(x, groupcount, as.num, val.labels, var.label, inclusive) {
@@ -99,12 +122,12 @@ split_var_helper <- function(x, groupcount, as.num, val.labels, var.label, inclu
                 include.lowest = !inclusive,
                 right = inclusive)
   # rename factor levels
-  levels(retval) <- c(1:groupcount)
+  levels(retval) <- seq_len(groupcount)
   # to numeric?
   if (as.num) retval <- to_value(retval)
   # set back variable and value labels
-  retval <- suppressWarnings(set_label(retval, var_lab))
-  retval <- suppressWarnings(set_labels(retval, val_lab))
+  retval <- suppressWarnings(set_label(retval, lab = var_lab))
+  retval <- suppressWarnings(set_labels(retval, labels = val_lab))
   # return value
   return(retval)
 }
