@@ -1,4 +1,4 @@
-#' @title Drop, add or convert (non-)labelled values to vectors
+#' @title Drop, add or convert (non-)labelled values
 #' @name zap_labels
 #'
 #' @description For (partially) labelled vectors, \code{zap_labels()} will replace
@@ -24,6 +24,11 @@
 #'           \item For \code{drop_labels()}, \code{x}, where value labels for non-existing values are removed.
 #'           \item For \code{fill_labels()}, \code{x}, where labels for non-labelled values are added.
 #'         }
+#'         If \code{x} is a data frame, the complete data frame \code{x} will be
+#'         returned, with variables specified in \code{...} being converted;
+#'         if \code{...} is not specified, applies to all variables in the
+#'         data frame.
+#'
 #'
 #' @examples
 #' # ------------------------
@@ -32,28 +37,32 @@
 #' data(efc)
 #' str(efc$e42dep)
 #'
-#' x <- set_labels(efc$e42dep, c("independent" = 1, "severe dependency" = 4))
+#' x <- set_labels(
+#'   efc$e42dep,
+#'   labels = c("independent" = 1, "severe dependency" = 4)
+#' )
 #' table(x)
 #' get_values(x)
 #' str(x)
 #'
 #' # zap all labelled values
-#' x <- set_labels(efc$e42dep, c("independent" = 1, "severe dependency" = 4))
 #' table(zap_labels(x))
 #' get_values(zap_labels(x))
 #' str(zap_labels(x))
 #'
 #' # zap all unlabelled values
-#' x <- set_labels(efc$e42dep, c("independent" = 1, "severe dependency" = 4))
 #' table(zap_unlabelled(x))
 #' get_values(zap_unlabelled(x))
 #' str(zap_unlabelled(x))
 #'
 #' # in a pipe-workflow
 #' library(dplyr)
-#' set_labels(efc$e42dep) <-  c("independent" = 1, "severe dependency" = 4)
 #' efc %>%
 #'   select(c172code, e42dep) %>%
+#'   set_labels(
+#'     e42dep,
+#'     labels = c("independent" = 1, "severe dependency" = 4)
+#'   ) %>%
 #'   zap_labels()
 #'
 #' # ------------------------
@@ -67,7 +76,7 @@
 #' # recode carers age into groups of width 5
 #' x <- rec(efc$c160age, recodes = rp$pattern)
 #' # add value labels to new vector
-#' set_labels(x) <- rp$labels
+#' x <- set_labels(x, labels = rp$labels)
 #'
 #' # watch result. due to recode-pattern, we have age groups with
 #' # no observations (zero-counts)
@@ -109,16 +118,9 @@ zap_labels <- function(x, ...) {
   .dots <- match.call(expand.dots = FALSE)$`...`
   .dat <- get_dot_data(x, .dots)
 
-  # get variable names
-  .vars <- dot_names(.dots)
-
-  # if user only provided a data frame, get all variable names
-  if (is.null(.vars) && is.data.frame(x)) .vars <- colnames(x)
-
-  # if we have any dot names, we definitely have a data frame
-  if (!is.null(.vars)) {
+  if (is.data.frame(x)) {
     # iterate variables of data frame
-    for (i in .vars) {
+    for (i in colnames(.dat)) {
       x[[i]] <- zap_labels_helper(.dat[[i]])
     }
     # coerce to tibble
@@ -139,16 +141,9 @@ zap_unlabelled <- function(x, ...) {
   .dots <- match.call(expand.dots = FALSE)$`...`
   .dat <- get_dot_data(x, .dots)
 
-  # get variable names
-  .vars <- dot_names(.dots)
-
-  # if user only provided a data frame, get all variable names
-  if (is.null(.vars) && is.data.frame(x)) .vars <- colnames(x)
-
-  # if we have any dot names, we definitely have a data frame
-  if (!is.null(.vars)) {
+  if (is.data.frame(x)) {
     # iterate variables of data frame
-    for (i in .vars) {
+    for (i in colnames(.dat)) {
       x[[i]] <- zap_unlabelled_helper(.dat[[i]])
     }
     # coerce to tibble
@@ -200,16 +195,9 @@ zap_na_tags <- function(x, ...) {
   .dots <- match.call(expand.dots = FALSE)$`...`
   .dat <- get_dot_data(x, .dots)
 
-  # get variable names
-  .vars <- dot_names(.dots)
-
-  # if user only provided a data frame, get all variable names
-  if (is.null(.vars) && is.data.frame(x)) .vars <- colnames(x)
-
-  # if we have any dot names, we definitely have a data frame
-  if (!is.null(.vars)) {
+  if (is.data.frame(x)) {
     # iterate variables of data frame
-    for (i in .vars) {
+    for (i in colnames(.dat)) {
       x[[i]] <- zap_na_tags_helper(.dat[[i]])
     }
     # coerce to tibble
@@ -263,17 +251,11 @@ zap_na_tags <- function(x, ...) {
 zap_inf <- function(x, ...) {
   # evaluate arguments, generate data
   .dots <- match.call(expand.dots = FALSE)$`...`
+  .dat <- get_dot_data(x, .dots)
 
-  # get variable names
-  .vars <- dot_names(.dots)
-
-  # if user only provided a data frame, get all variable names
-  if (is.null(.vars) && is.data.frame(x)) .vars <- colnames(x)
-
-  # if we have any dot names, we definitely have a data frame
-  if (!is.null(.vars)) {
+  if (is.data.frame(x)) {
     # iterate variables of data frame
-    for (i in .vars) {
+    for (i in colnames(.dat)) {
       # convert NaN and Inf to missing
       x[[i]][is.nan(x[[i]])] <- NA
       x[[i]][is.infinite(x[[i]])] <- NA
@@ -291,19 +273,19 @@ zap_inf <- function(x, ...) {
 
 
 zap_labels_helper <- function(x) {
-  x <- set_na(x, get_values(x, drop.na = T))
+  x <- set_na(x, value = get_values(x, drop.na = T))
   # auto-detect variable label attribute
   attr.string <- getVarLabelAttribute(x)
   # remove label attributes
   if (!is.null(attr.string)) attr(x, attr.string) <- NULL
-  if (haven::is.labelled(x)) class(x) <- NULL
+  if (is_labelled(x)) class(x) <- NULL
   return(x)
 }
 
 zap_unlabelled_helper <- function(x) {
   vals <- get_values(x)
-  x <- set_na(x, stats::na.omit(unique(x)[!unique(x) %in% vals]))
-  if (haven::is.labelled(x)) class(x) <- NULL
+  x <- set_na(x, value = stats::na.omit(unique(x)[!unique(x) %in% vals]))
+  if (is_labelled(x)) class(x) <- NULL
   return(x)
 }
 
@@ -311,6 +293,5 @@ zap_na_tags_helper <- function(x) {
   # convert all NA, including tagged NA, into regular NA
   x[is.na(x)] <- NA
   # "remove" labels from tagged NA values
-  set_labels(x) <- get_labels(x, attr.only = T, include.values = "n", drop.na = T)
-  return(x)
+  set_labels(x, labels = get_labels(x, attr.only = T, include.values = "n", drop.na = T))
 }
